/*
------------------------------------
create_schema_edt.sql 
------------------------------------

L'utilisation des rfrences permet de garantir que les contraintes 
lies aux dpendances fonctionnelles sont respectes
*/

-- Suppresion des tables
drop table affichage;
drop table edt_sem;
drop table cursus_creneau_type;
drop table cursus;
drop table creneau_type;
drop table module;
drop table edt_init;
drop table creneau_init;

-------------------------------------
-- Tables de donnees
-- ----------------------------------

create table creneau_init(
    code_module  varchar(20),
    libelle varchar(50),
    cursus  varchar(10),
    nomcr   varchar(10),
    duree   numeric(1)
);

create table edt_init (
    module  varchar(20),
    nomcr   varchar(10),    
    sem     numeric(2),
    jour    numeric(1),
    heured  numeric(1),
    salle   varchar(10)
);


-- ----------------------------------
-- Table pour le stockage de EDT
-- ----------------------------------


create table module (                       -- un module est dfini par son code
    code_module  varchar(20) primary key,
    libelle varchar(50)
);

create table creneau_type (                 -- chaque module(code_module,nomcr) a une dure

    code_module  varchar(20) references module,
    nomcr   varchar(10),
    duree   numeric(1),
    primary key (code_module, nomcr)
);

create table cursus (
    nomcursus   varchar(10) primary key
);

create table cursus_creneau_type (           -- lien entre cursus et crneau type
    nomcursus  varchar(10) references cursus,
    code_module  varchar(20) references module,
    nomcr   varchar(10),
    foreign key (code_module, nomcr) references creneau_type,
    PRIMARY KEY  (nomcursus, code_module, nomcr)
);

create table edt_sem (
    code_module  varchar(20) references module,
    nomcr   varchar(10),    
    sem     numeric(2) CHECK ( sem >= 1 and sem <= 52),
    jour    numeric(1) CHECK ( jour  >= 1 and jour <= 5),
    heured  numeric(1) CHECK ( heured >= 1 and heured <= 8),
    salle   varchar(10),
      primary key (code_module, nomcr, sem), -- code_module, nomcr, sem -> jour, heured, dure
      unique (sem, jour, heured, salle ),    -- il n'y a pas 2 cours en mme temps dans la mme salle  
      CONSTRAINT  c1  foreign key (code_module, nomcr) references creneau_type  (code_module, nomcr)
);

/*
-------------------------------------------------
Table pour l'affichage de EDT d'une semaine
-------------------------------------------------
*/



create table affichage (
    code_module  varchar(20), -- ne peut rfrencer module car devra prendre la valeur nulle
                              -- pour les crneaux vides
    nomcr  varchar(10),
    jour    numeric(1) ,
    heured  numeric(2) , 
    duree  numeric(2),
    ligne   numeric(2),
    salle   varchar(10),
--    unique (code_module, nomcr),
    primary key (jour,ligne, heured)
);


/*
-------------------------------------------------
Attribution des droits (necessaires pour l'accs
via le server resin)
-------------------------------------------------
*/


grant select on 
    module, creneau_type, cursus, cursus_creneau_type,  edt_sem
to  invited;

grant select, insert, delete, update on affichage to invited;


